require 'find'
require 'rss/0.9'
require 'mp3info'

module Podcast 

    class Feed

        attr_reader :title, :link, :description, :image, :base, :language
        attr_writer :title, :link, :description, :image, :base, :language

        include Enumerable

        def initialize
            @mp3s = []
            @language = "English"
            @base = ''
        end

        # add an mp3 file to the podcast
        def add_mp3( file )
            mp3 = Mp3File.new( file )
            @mp3s.push( mp3 )
        end

        # add a directory location to the podcast
        # the directory will be recursively search
        # for mp3 files.
        def add_dir( dir )
            # we chdir into the directory so that file paths will be relative
            pwd = Dir.pwd
            Dir.chdir( dir )
            Find.find( '.' ) do |f|
                if ( f =~ /\.mp3$/ )
                    f.sub!( %r{^./}, '' ) # remove leading './'
                    add_mp3(f)
                end
            end
            # go back to original directory
            Dir.chdir( pwd )
        end

        # the total amount of files in the podcast
        def size
            @mp3s.size
        end

        # write the podcast file 
        def get_rss()
            rss = RSS::Rss.new( "0.9" )
            channel = RSS::Rss::Channel.new
            channel.title = @title
            channel.description = @description 
            channel.link = @link 
            channel.language = @language
            rss.channel = channel

            image = RSS::Rss::Channel::Image.new
            image.url = @image
            image.title = @title
            image.link = @link
            channel.image = image

            for mp3 in @mp3s 
                next if ! mp3.artist
                item = RSS::Rss::Channel::Item.new
                item.title = mp3
                ## add a base url 
                if base != ''
                    link = base + '/' + mp3.path
                else 
                    link = mp3.path
                end
                item.link = link
                item.enclosure = RSS::Rss::Channel::Item::Enclosure.new( 
                    link, mp3.length, mp3.type )
                channel.items << item
            end

            return rss.to_s
        end

        def each
            @songs.each do |s|
                yield s
            end
        end

    end

    class Mp3File

        attr_reader :artist, :album, :title, :file, :path, :length, :type, :mtime
        attr_writer :artist, :album, :title, :file, :path, :length, :type, :mtime

        def initialize(f)
            file = File.new(f)
            info = Mp3Info.open(f)
            tag = info.tag()
            @file = f
            @artist = tag['artist']
            @album = tag['album']
            @title = tag['title']
            @path = file.path()
            @length = file.stat.size()
            @mtime = file.stat.mtime()
            @type = 'audio/mpeg'
        end

        def to_s
            "#{@artist} -- #{@album} -- #{@title}"
        end

    end

end
